@@ -115,6 +115,9 @@ PluginManager::~PluginManager()
     // inform all plugins and client windows that we close now
     emit sigClosed();
 
+    // wait until all plugins are really closed
+    this->sync();
+
     // release all unique plugins
     QPtrListIterator<KwavePlugin> itu(m_unique_plugins);
     for (itu.toLast() ; itu.current(); ) {
@@ -382,28 +385,23 @@ int PluginManager::executePlugin(const QString &name, QStringList *params)
     return result;
 }
 
+//***************************************************************************
+bool PluginManager::onePluginRunning()
+{
+    QPtrListIterator<KwavePlugin> it(m_loaded_plugins);
+    for (; it.current(); ++it) {
+	KwavePlugin *plugin = it.current();
+	if (plugin->isRunning()) return true;
+    }
+    return false;
+}
+
 //***************************************************************************
 void PluginManager::sync()
 {
-    bool one_is_running = true;
-    while (one_is_running) {
-	QPtrListIterator<KwavePlugin> it(m_loaded_plugins);
-	one_is_running = false;
-
-	for (; it.current(); ++it) {
-	    KwavePlugin *plugin = it.current();
-	    if (plugin->isRunning()) {
-		qDebug("waiting for plugin '%s'",
-		       plugin->name().local8Bit().data());
-		one_is_running = true;
-		break;
-	    }
-	}
-
-	if (one_is_running) {
-	    qApp->processEvents();
-	    sleep(1000);
-	}
+    while (onePluginRunning()) {
+	qApp->processEvents();
+	pthread_yield();
     }
 }
 
@@ -632,6 +630,12 @@ void PluginManager::forwardCommand()
     }
 }
 
+//***************************************************************************
+void PluginManager::signalClosed()
+{
+    emit sigClosed();
+}
+
 //***************************************************************************
 void PluginManager::pluginClosed(KwavePlugin *p)
 {
@@ -786,7 +790,6 @@ void PluginManager::registerPlaybackDeviceFactory(
     PlaybackDeviceFactory *factory)
 {
     m_playback_factories.append(factory);
-
 }
 
 //****************************************************************************
