@@ -40,6 +40,7 @@
 #include "libkwave/SampleReader.h"
 #include "libkwave/SampleWriter.h"
 #include "libkwave/Signal.h"
+#include "libkwave/Track.h"
 
 #include "libgui/MenuManager.h"
 #include "libgui/TrackPixmap.h"
@@ -421,7 +422,7 @@ int SignalWidget::loadFile(const KURL &url)
 		reason = i18n("unable to open '%1'").arg(
 		    url.prettyURL());
 		break;
-	    case -EMEDIUMTYPE:
+	    case -EINVAL:
 		reason = i18n("invalid or unknown file type: '%1'").arg(
 		              url.prettyURL());
 		break;
@@ -734,7 +735,6 @@ void SignalWidget::allowRepaint(bool repaint)
 	    m_repaint_timer.start(REPAINT_INTERVAL, true);
 	}
     }
-
 }
 
 //***************************************************************************
@@ -966,13 +966,11 @@ void SignalWidget::mouseMoveEvent(QMouseEvent *e)
     }
 }
 
-//***************************************************************************
 void SignalWidget::paintEvent(QPaintEvent *)
 {
     InhibitRepaintGuard inhibit(*this, false); // avoid recursion
-    QPainter p;
 
-//    qDebug("SignalWidget::paintEvent()");
+//     qDebug("SignalWidget::paintEvent()");
 //#ifdef DEBUG
 //    static struct timeval t_start;
 //    static struct timeval t_end;
@@ -990,7 +988,7 @@ void SignalWidget::paintEvent(QPaintEvent *)
     m_width = QWidget::width();
     m_height = QWidget::height();
 
-//    qDebug("SignalWidget::paintEvent(): width=%d, height=%d",m_width,m_height);
+//     qDebug("SignalWidget::paintEvent(): width=%d, height=%d",m_width,m_height);
 
     // --- detect size changes and refresh the whole display ---
     if ((m_width != m_last_width) || (m_height != m_last_height)) {
@@ -1019,12 +1017,10 @@ void SignalWidget::paintEvent(QPaintEvent *)
 	Q_ASSERT(m_layer[LAYER_SIGNAL]);
 	if (!m_layer[LAYER_SIGNAL]) return;
 
-//	qDebug("SignalWidget::paintEvent(): - redraw of signal layer -");
-	p.begin(m_layer[LAYER_SIGNAL]);
-	p.setRasterOp(CopyROP);
+// 	qDebug("SignalWidget::paintEvent(): - redraw of signal layer -");
 
 	// all black if empty
-	if (!n_tracks) p.fillRect(0, 0, m_width, m_height, black);
+	if (!n_tracks) m_layer[LAYER_SIGNAL]->fill(black);
 
 	int track_height = (n_tracks) ? (m_height / n_tracks) : 0;
 	int top = 0;
@@ -1038,7 +1034,6 @@ void SignalWidget::paintEvent(QPaintEvent *)
 		pix->resize(m_width, track_height);
 	    }
 	    if (pix->isModified()) {
-//		qDebug("SignalWidget::paintEvent(): track %d",i); // ###
 		pix->repaint();
 	    }
 
@@ -1048,8 +1043,6 @@ void SignalWidget::paintEvent(QPaintEvent *)
 	    top += track_height;
 	}
 
-	p.end();
-
 	m_update_layer[LAYER_SIGNAL] = false;
 	update_pixmap = true;
     }
@@ -1062,12 +1055,11 @@ void SignalWidget::paintEvent(QPaintEvent *)
 	if (!m_layer[LAYER_MARKERS]) return;
 
 //	qDebug("SignalWidget::paintEvent(): - redraw of markers layer -");
-	p.begin(m_layer[LAYER_MARKERS]);
-	p.fillRect(0, 0, m_width, m_height, black);
+	m_layer[LAYER_MARKERS]->fill(black);
 
+// 	p.begin(m_layer[LAYER_MARKERS]);
 	// ### nothing to do yet
-
-	p.end();
+// 	p.end();
 
 	m_update_layer[LAYER_MARKERS] = false;
 	update_pixmap = true;
@@ -1080,11 +1072,12 @@ void SignalWidget::paintEvent(QPaintEvent *)
 	Q_ASSERT(m_layer[LAYER_SELECTION]);
 	if (!m_layer[LAYER_SELECTION]) return;
 
-//	qDebug("SignalWidget::paintEvent(): - redraw of selection layer -");
+// 	qDebug("SignalWidget::paintEvent(): - redraw of selection layer -");
 
-	p.begin(m_layer[LAYER_SELECTION]);
-	p.fillRect(0, 0, m_width, m_height, black);
-	p.setRasterOp(CopyROP);
+	m_layer[LAYER_SELECTION]->fill(black);
+
+	QPainter p;
+ 	p.begin(m_layer[LAYER_SELECTION]);
 
 	if (n_tracks) {
 	    unsigned int left  = m_signal_manager.selection().first();
@@ -1120,8 +1113,12 @@ void SignalWidget::paintEvent(QPaintEvent *)
 	if (!m_pixmap) return;
 	update_pixmap = true;
     }
+    Q_ASSERT(m_pixmap->width() == m_width);
+    Q_ASSERT(m_pixmap->height() == m_height);
 
+    // bitBlt all layers together
     if (update_pixmap) {
+	m_pixmap->fill(black);
 	for (int i=0; i < 3; i++) {
 	    if (!m_layer[i]) continue;
 	    bitBlt(m_pixmap, 0, 0, m_layer[i], 0, 0,
@@ -1135,6 +1132,7 @@ void SignalWidget::paintEvent(QPaintEvent *)
 	m_signal_manager.playbackController().currentPos() - m_offset);
 
     if (n_tracks) {
+	QPainter p;
 	p.begin(m_pixmap);
 	p.setPen(yellow);
 	p.setRasterOp(XorROP);
@@ -1747,7 +1745,13 @@ void SignalWidget::slotTrackInserted(unsigned int index, Track &track)
     if (pix->isModified()) refreshSignalLayer();
 
     // connect all signals
-    connect(pix, SIGNAL(sigModified()), this, SLOT(refreshSignalLayer()));
+    connect(pix, SIGNAL(sigModified()),
+            this, SLOT(refreshSignalLayer()));
+
+    connect(&track, SIGNAL(sigSelectionChanged()),
+	    this, SIGNAL(sigTrackSeclecionChanged()));
+    connect(&track, SIGNAL(sigSelectionChanged()),
+            this, SLOT(refreshSignalLayer()));
 }
 
 //***************************************************************************
