@@ -59,6 +59,7 @@ class TrackPixmap : public QObject, public QPixmap
     Q_OBJECT
 
 public:
+
     /** Default constructor */
     TrackPixmap(Track &track);
 
@@ -86,6 +87,9 @@ public:
      */
     bool isModified();
 
+    /** Sets the internal "modified" flag */
+    void setModified() { m_modified = true; };
+
 signals:
 
     /** Emitted if the content of the pixmap was modified. */
@@ -143,6 +147,31 @@ private slots:
     void slotSamplesModified(Track &src, unsigned int offset,
                              unsigned int length);
 
+    /**
+     * Sets the state of the pixmap to "modified" whenever the
+     * selection of the track has changed
+     */
+    void selectionChanged();
+
+public:
+
+    typedef struct {
+	/** Background color */
+	QColor background;
+
+	/** Color for samples */
+	QColor sample;
+
+	/** Color for interpolated samples */
+	QColor interpolated;
+
+	/** Color for the zero line, used areas */
+	QColor zero;
+
+	/** Color of the zero line, unused areas */
+	QColor zero_unused;
+    } color_set_t;
+
 private:
 
     /**
@@ -309,20 +338,8 @@ private:
      */
     float *m_interpolation_alpha;
 
-    /** Background color */
-    QColor m_color_background;
-
-    /** Color for samples */
-    QColor m_color_sample;
-
-    /** Color for interpolated samples */
-    QColor m_color_interpolated;
-
-    /** Color for the zero line, used areas */
-    QColor m_color_zero;
-
-    /** Color of the zero line, unused areas */
-    QColor m_color_zero_unused;
+    /** set of colors for drawing */
+    color_set_t m_colors;
 
 };
 
