@@ -350,6 +350,14 @@ void Track::deleteRange(unsigned int offset, unsigned int length,
     emit sigSamplesDeleted(*this, offset, length);
 }
 
+//***************************************************************************
+void Track::select(bool selected)
+{
+    if (m_selected == selected) return;
+    m_selected = selected;
+    emit sigSelectionChanged();
+}
+
 //***************************************************************************
 void Track::slotSamplesInserted(Stripe &src, unsigned int offset,
                                 unsigned int length)
