@@ -65,8 +65,7 @@ PlayBackALSA::PlayBackALSA()
     m_buffer_size(0),
     m_buffer_used(0),
     m_format(),
-    m_chunk_size(0)//,
-//     m_device_list()
+    m_chunk_size(0)
 {
 }
 
@@ -429,7 +428,8 @@ QString PlayBackALSA::open(const QString &device, double rate,
 		break;
 	    default:
 		reason = i18n("Opening the device '%1' failed: %2").arg(
-	            device).arg(QString::fromLocal8Bit(snd_strerror(err)));
+	            device.section('|',0,0)).arg(
+		    QString::fromLocal8Bit(snd_strerror(err)));
 	}
 	return reason;
     }
@@ -465,6 +465,7 @@ int PlayBackALSA::write(QMemArray<sample_t> &samples)
     }
 
     // convert into byte stream
+    /* ### TODO handling for big endian machines ### */
     unsigned int channel;
     for (channel=0; channel < m_channels; channel++) {
 	sample_t sample = samples[channel];
@@ -515,7 +516,7 @@ int PlayBackALSA::flush()
 	unsigned int buffer_samples = m_buffer_size / m_bytes_per_sample;
 	unsigned int timeout = (m_rate > 0) ?
 	    3 * ((1000 * buffer_samples) / (unsigned int)m_rate) : 1000U;
-	unsigned char *p = reinterpret_cast<unsigned char *>(m_buffer.data());
+	u_int8_t *p = reinterpret_cast<u_int8_t *>(m_buffer.data());
 	int r;
 
 	// pad the buffer with silence if necessary
@@ -734,7 +735,6 @@ QStringList PlayBackALSA::supportedDevices()
     QStringList list;
 
     // re-validate the list if necessary
-//     alsaDeviceName(m_device_name);
     scanDevices();
 
     QMap<QString, QString>::Iterator it;
@@ -844,7 +844,6 @@ int PlayBackALSA::detectChannels(const QString &device,
     // close the device if *we* opened it
     if (pcm != m_handle) snd_pcm_close(pcm);
 
-//     qDebug("PlayBackALSA::detectTracks, min=%u, max=%u", min, max);
     return 0;
 }
 
